<?php
/*
ftslocallogviewer.php
Copyright (C) 2008,2009,2010 IN2P3, CNRS

This file is part of Ftsmonitor.

Ftsmonitor is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ftsmonitor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ftsmonitor; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
* ftslocallogviewer.php
*
* Plugin for reading log files on a local or remote file system.
* 
* @version '$Id: ftslocallogviewer.php,v 1.5 2010/06/28 14:12:54 schwarz Exp $'
* @author Lionel Schwarz <lionel.schwarz@in2p3.fr>
*/

require_once('ftslog.classes.php');

/**
 * This class allows to read a log file located on a local or remote file system.
 *
 * @package TransferLog
 * @see FTSLogViewer
 * @see FTSLog
 */
class FTSLocalLogViewer extends FTSLogViewer {

  /**
  * Constructor
  * @param string $root : the root path of the file to read ('/var/tmp/' by default)
  */
  public function __construct($root='/var/tmp/') {
    parent::__construct($root);
  }

  /**
  * This method reads the log in a file located in the local file system
  *
  * The root directory <ROOT> is defined in the configuration file. It is "/var/tmp" by default
  * The relative log file will be:
  * - if transfer is in state 'Processing': "<ROOT>/glite-url-copy-edguser/<IDENTIFIER>.log"
  * - if transfer is in state 'Aborted':  "<ROOT>/glite-url-copy-edguser/<CHANNEL>failed/<IDENTIFIER>.log"
  * - otherwise: "<ROOT>/glite-url-copy-edguser/<CHANNEL><STATE_IN_LOWERCASE>/<IDENTIFIER>.log"
  *
  * Or the log file can be found in an archive named like:
  * - if transfer is in state 'Aborted' or 'Failed':  "<ROOT>/transfer_failed/glite-[url|srm]-copy-<CHANNEL>failed-<DATEHOUR>-\d+.tar.gz"
  * - if transfer is in state 'Completed': <ROOT>/transfer_completed/glite-[url|srm]-copy-completed-<DATEHOUR>-\d+.tar.gz"
  *
  * @return Array : an array with each row containg a line of the log file
  * @see FTSLog
  */
  public function retrieveLog() {
    // first search non-archived logs
    $glitedir = "glite-url-copy-edguser";
    if ($this->log->transfer_state == FTS_STATE_PROCESSING) {
      $logfilename = sprintf("%s/%s.log", $glitedir, $this->log->transfer_id);
    } else {
      $logfilename = sprintf("%s/%s",	$glitedir, $this->log->getArchivedFileName());
    }

    $logfilename = $this->root . '/' . $logfilename;
       ftsSyslog(sprintf("Logfile is %s", $logfilename), LOG_WARNING);
   
    if (file_exists($logfilename)) {
      return file($logfilename);
    } else {
      // search for archives
      if ($this->log->transfer_state == FTS_STATE_ABORTED) 
        $archdir = "transfer_failed/";
      else
        $archdir = sprintf("transfer_%s/", strtolower($this->log->transfer_state));
      $archdir = $this->root . '/' . $archdir;
      // pattern: glite-[url|srm]-copy-[<CHANNEL>]{0,1}<STATE>-<DATEHOUR>-\d+.tar.gz
      // for example: "glite-url-copy-LPNHE-IN2P3failed-1005300100-24.tar.gz"
      //           Or "glite-url-copy-completed-1005300100-1.tar.gz"
      if ($this->log->transfer_state == FTS_STATE_ABORTED or $this->log->transfer_state == FTS_STATE_FAILED) 
        $filePattern = sprintf("glite-*-copy-%s%s-*.tar.gz", 
				$this->log->getChannel(), 
				"failed");
      else // Completed: the channel is not in the name
        $filePattern = sprintf("glite-*-copy-%s-*.tar.gz", 
				strtolower($this->log->transfer_state));

      foreach (glob($archdir . $filePattern) as $f) {
        if (filemtime($f) > $this->log->getTransferDate()) {
          $lines = file('phar://' . $f . '/' . $this->log->getArchivedFileName());
          if ($lines != Null) return $lines;
        }
      }
    }
    throw new Exception("Could not find LOG $logfilename nor archived log in $archdir/$filePattern");
  }
}

