<?php
/*
ftslog.classes.php
Copyright (C) 2008,2009,2010 IN2P3, CNRS

This file is part of Ftsmonitor.

Ftsmonitor is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ftsmonitor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ftsmonitor; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
* ftslog.classes.php
*
* This file contains generic classes for transfers logs
* 
* @version '$Id: ftslog.classes.php,v 1.4 2010/06/28 14:12:54 schwarz Exp $';
* @author Lionel Schwarz <lionel.schwarz@in2p3.fr>
*/

require_once('ftsconstants.php');

/**
* A class for FTS transfer log
*
* Represented as an array with each line of the log in a row
* @package TransferLog
* @see FTSLogViewer
*/
class FTSLog extends ArrayObject {
  /**
  * Identifier of the transfer log
  *
  * Something like "<CHANNEL_NAME>__<DATE>_XXXXXX"
  * defined in the DB as "T_TRANSFER.TRANSFER_ID"
  * @var string
  * @see getTransferId
  */
  var $transfer_id;

  /**
  * Host where the log file was originally written
  *
  * In case of multiple hosts, transfer logs are distributed depending on where agents are running
  * @var string
  * @see getHost
  */
  var $host;

  /**
  * State of the transfer
  *
  * can be "Completed" or "Failed"
  * defined in the DB as "T_TRANSFER_TRANSFER_STATE"
  * @var string
  * @see getTransferState
  */
  var $transfer_state;

  /**
  * Constructor
  *
  * @param string $host Host : where the log was written
  * @param string $transfer_id : Identifier of the transfer
  * @param string $transfer_state : State of the transfer
  * @param array $array : an array containing one log line in each row
  */
  public function __construct($host, $transfer_id, $transfer_state, $array=null) {
    $this->host = $host;
    $this->transfer_id = $transfer_id;
    $this->transfer_state = $transfer_state;
    parent::__construct(Array()); //$array);
  }

  /**
  * Gets the identifier of the transfer
  * 
  * @return string : the identifier of the transfer as found in the DB in "T_TRANSFER.TRANSFER_ID"
  */
  public function getTransferID() {
    return $this->transfer_id;
  }

  /**
  * Gets the state of the transfer
  * 
  * @return string : the state of the transfer as found in the DB in "T_TRANSFER.TRANSFER_STATE"
  */
  public function getTransferState() {
    return $this->transfer_state;
  }

  /**
  * Gets the host of the transfer
  * 
  * @return string : the host where the transfer log was originally written to
  */
  public function getHost() {
    return $this->host;
  }

  /**
  * Gets the name of the log file
  * 
  * @return string the log file name built as: "<IDENTIFIER>.log"
  */
  public function getName() {
    return $this->transfer_id . '.log';
  }

  /**
  * Gets the archive file name
  *
  * The archive file name is built as follows:
  * - if transfer is "Aborted": "<CHANNEL_NAME>failed/<IDENTIFIER>.log"
  * - if the transfer is "Completed" or "Failed": "<CHANNEL_NAME><STATE_IN_LOWERCASE>/<IDENTIFIER>.log"
  * @return string : the archive file name
  */ 
  public function getArchivedFileName() {
	if ($this->transfer_state == FTS_STATE_ABORTED) $this->transfer_state = FTS_STATE_FAILED;
	//list($channel, $dummy) = explode('__', $this->transfer_id);
	$logfile = sprintf("%s%s/%s.log",
				$this->getChannel(),
				strtolower($this->transfer_state),
				$this->transfer_id);
    return $logfile;
  }

  /**
  * Gets the Transfer date
  *
  * The date is included in the transfer identifier in the form "YYYY-MM-DD-HHmm"
  * @return integer : the transfer date as a timestamp
  */
  public function getTransferDate() {
	list($channel, $dummy) = explode('__', $this->transfer_id);
	list($when, $dummy) = explode('_', $dummy);
	// now we got $when = something like '2010-06-08-2254'
	$format = '%Y-%m-%d-%H%M';
	$date_array = strptime($when, $format);
        // Array ( [tm_sec] => 0 [tm_min] => 54 [tm_hour] => 22 [tm_mday] => 8 [tm_mon] => 5 [tm_year] => 110 [tm_wday] => 2 [tm_yday] => 158
	return mktime ($date_array['tm_hour'],
			$date_array['tm_min'],
			0, // PHP5.1 has a bug on 'tm_sec'
			$date_array['tm_mon']+1,
			$date_array['tm_mday'],
			$date_array['tm_year']+1900);

  }

  /**
  * Gets the channel that managed the transfer
  *
  * @return string : the channel extracted from the identifier
  */
  public function getChannel() {
    list($channel, $dummy) = explode('__', $this->transfer_id);
    return $channel;
  }

  /**
  * Gets the HTML display
  *
  * @return string : the HTML table containing the log with each of its line in a row
  */
  public function toHTML() {
    $html = '<table id="alerts">';
    $iterator = $this->getIterator();

    while($iterator->valid()) {
     if (preg_match('/\[ERROR *\]/',$iterator->current())) {
       $html .= sprintf('<tr class="Error"><td>');
     } else if (preg_match('/\[WARN *\]/',$iterator->current())) {
       $html .= sprintf('<tr class="Warning"><td>');
     } else {
       $html .= sprintf('<tr class="OK"><td>');
     }
     $html .= htmlspecialchars($iterator->current());
     $html .= '</td></tr>';
     $iterator->next();
    }
    $html .= "</table>";
    return $html;
  }
}

/**
* This class is the generic class for log viewers
*
* You have to define your own viewer or use viewers provided by default
* @see FTSHTTPLogViewer
* @see FTSLocalLogViewer
* @package TransferLog
*/
abstract class FTSLogViewer {

  /**
  * Root path of the file/URL to read
  * @var string
  */
  var $root;

  /**
  * The FTSLog object that contains lines of the log and attributes
  * @var FTSLog
  */
  var $log;

  /**
  * Constructor
  * @param string $root : the root path of the file to read
  */
  public function __construct($root) {
    $this->root = $root;
  }
  

  /*public function getName($transfer_id, $transfer_state) {
    return $this->getArchivedFileName($transfer_id, $transfer_state);
  }*/

  /**
  * Retrives the log as an array
  *
  * This method has to be implemented depending on the way the log is retrieved
  * @return array : an array with each log line in a row
  */
  public abstract function retrieveLog();

  /**
  * Gets the LOG object built from the array returned by "getLogAsArray"
  * @param string $host : host where the transfer log can be retrieved
  * @param string $transfer_id : identifier of the transfer
  * @param string $transfer_state : state of the transfer
  * @return FTSLog : the log as an object or NULL is there is an error
  * @see FTSLog
  * @see retrieveLog
  */
  public function getLog($host, $transfer_id, $transfer_state) {
    /*$res_array = $this->getLogAsArray($host, $transfer_id, $transfer_state);
    if (! is_null($res_array)) {
      return new FTSLog($host, $transfer_id, $transfer_state, $res_array);
    }
    return Null;*/
    $this->log = new FTSLog($host, $transfer_id, $transfer_state);
    //$res_array = $this->retrieveLog();
    foreach ($this->retrieveLog() as $logline) {
      $this->log->append($logline);
    }
    return $this->log;
  }
}

